#!/usr/bin/env python
"""
_ListFilesOverInterval_

Retrieve meta data about express files belonging to a particular dataset
and produced over a certain interval.
"""

__revision__ = "$Id: ListFilesOverInterval.py,v 1.4 2011/07/19 12:58:34 hufnagel Exp $"
__version__ = "$Revision: 1.4 $"

import time

from WMCore.Database.DBFormatter import DBFormatter

class ListFilesOverInterval(DBFormatter):

    sql = """SELECT wmbs_file_details.lfn AS lfn,
                    wmbs_file_details.filesize AS file_size,
                    wmbs_file_details.events AS events,
                    wmbs_file_runlumi_map.run AS run,
                    wmbs_file_runlumi_map.lumi AS lumi
              FROM wmbs_fileset
              INNER JOIN wmbs_fileset_files ON
                wmbs_fileset_files.fileset = wmbs_fileset.id AND
                wmbs_fileset_files.insert_time >= :START_TIME AND
                wmbs_fileset_files.insert_time <= :END_TIME
              INNER JOIN wmbs_file_runlumi_map ON
                wmbs_file_runlumi_map.fileid = wmbs_fileset_files.fileid
              INNER JOIN wmbs_file_details ON
                wmbs_file_details.id = wmbs_fileset_files.fileid
              INNER JOIN wmbs_file_dataset_path_assoc ON
                wmbs_file_dataset_path_assoc.file_id = wmbs_fileset_files.fileid
              INNER JOIN dataset_path ON
                dataset_path.id = wmbs_file_dataset_path_assoc.dataset_path_id
              WHERE wmbs_fileset.name = 'ExpressDBSUploadable'
              AND dataset_path.primary_dataset =
                (SELECT id FROM primary_dataset WHERE name = :PRIMDS)
              AND dataset_path.processed_dataset =
                (SELECT id FROM processed_dataset WHERE name = :PROCDS)
              AND dataset_path.data_tier =
                (SELECT id FROM data_tier WHERE name = :DATATIER)
              """

    def format(self, results):
        """
        _format_

        Format the result of the query into a dictionary for each file and
        return a list of dictionaries.  We get a row back for each run/lumi
        combination. 
        """
        results = self.formatDict(results)

        resultsDict = {}
        for result in results:
            if resultsDict.has_key(result["lfn"]):
                runDict = resultsDict[result["lfn"]]["runs"]
                if runDict.has_key(str(result["run"])):
                    runDict[str(result["run"])].append(int(result["lumi"]))
                else:
                    runDict[str(result["run"])] = [int(result["lumi"])]
            else:
                run = {str(result["run"]): [int(result["lumi"])]}
                resultsDict[result["lfn"]] = {"lfn": result["lfn"],
                                              "file_size": int(result["file_size"]),
                                              "events": int(result["events"]),
                                              "runs": run}
        resultsList = []        
        for lfn in resultsDict.keys():
            resultsList.append(resultsDict[lfn])

        return resultsList

    def execute(self, startTime, primaryDataset, processedDataset, dataTier,
                conn = None, transaction = False):
        """
        _execute_

        Figure out the end time and run the query.  We take a second off the
        end time to assure ourselves that we don't miss any files that may get
        added later this second.

        Create a dictionary that contains the files, start_time, end_time and
        dataset that was queried.

        """
        endTime = int(time.time()) - 1

        bindVars = { 'START_TIME' : startTime,
                     'END_TIME' : endTime,
                     'PRIMDS' : primaryDataset,
                     'PROCDS' : processedDataset,
                     'DATATIER' : dataTier }

        result = self.dbi.processData(self.sql, bindVars, conn = conn,
                                      transaction = transaction)

        resultDict = {}
        resultDict["files"] = self.format(result)
        resultDict["dataset_path"] = "/%s/%s/%s" % (primaryDataset,
                                                    processedDataset,
                                                    dataTier)
        resultDict["start_time"] = startTime
        resultDict["end_time"] = endTime
        
        return resultDict
